@@ -7,8 +7,18 @@ from multiprocessing import Process, Manager
 import datetime
 import subprocess
 import json
+import re
 
-version = '3.53'
+    
+numpy_available = False
+try:
+    from numpy.random import Generator, SFC64
+    numpy_available = True
+except:
+    numpy_available = False
+
+version = '3.61.0'
+__version__ = version
 
 # --------------------------------
 # TeeLogger Inline print only
@@ -45,92 +55,187 @@ def printWithColor(msg, level = 'info'):
     else:
         print(bcolors.info + msg + bcolors.ENDC)
 try:
-    from Tee_Logger import teeLogger
+    import Tee_Logger
+    assert float(Tee_Logger.version) >= 6.34
 except:
-    class teeLogger:
-        def __init__(self, systemLogFileDir='.', programName='iotest', compressLogAfterMonths=2, deleteLogAfterYears=2, suppressPrintout=False, fileDescriptorLength=15,noLog=True):
-            self.name = programName
-            self.currentDateTime = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
-            self.noLog = True
-            if not noLog:
-                print('Using inline logger, foring noLog...')
-            self.systemLogFileDir = '/dev/null'
-            self.logsDir = '/dev/null'
-            self.logFileDir = '/dev/null'
-            self.logFileName = '/dev/null'
-            self.compressLogAfterMonths = compressLogAfterMonths
-            self.deleteLogAfterYears = deleteLogAfterYears
-            self.suppressPrintout = suppressPrintout
-            self.fileDescriptorLength = fileDescriptorLength
-            self.version = '0.1 inline'
+    class Tee_Logger:
+        version = '0.1 inline'
+        class teeLogger:
+            def __init__(self, systemLogFileDir='.', programName=None, compressLogAfterMonths=2, 
+                 deleteLogAfterYears=2, suppressPrintout=..., fileDescriptorLength=15,
+                 noLog=False,callerStackDepth=-2,disable_colors=False, encoding = None,
+                 in_place_compression = None, collapse_single_day_logs = ...,compression_level=...,
+                 binary_mode = True):
+                self.name = programName
+                self.currentDateTime = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
+                self.noLog = True
+                if not noLog:
+                    print('Using inline logger, foring noLog...')
+                self.systemLogFileDir = '/dev/null'
+                self.logsDir = '/dev/null'
+                self.logFileDir = '/dev/null'
+                self.logFileName = '/dev/null'
+                self.compressLogAfterMonths = compressLogAfterMonths
+                self.deleteLogAfterYears = deleteLogAfterYears
+                self.suppressPrintout = suppressPrintout
+                self.fileDescriptorLength = fileDescriptorLength
+                self.version = Tee_Logger.version
 
-            self.logger = None
+                self.logger = None
 
-        def log_with_caller_info(self, level, msg):
-            return
+            def log_with_caller_info(self, level, msg):
+                return
 
-        def teeok(self, msg):
-            if not self.suppressPrintout:
-                printWithColor(msg, 'okgreen')
-            self.log_with_caller_info('info', msg)
+            def teeok(self, msg):
+                if not self.suppressPrintout:
+                    printWithColor(msg, 'okgreen')
+                self.log_with_caller_info('info', msg)
 
-        def ok(self, msg):
-            self.log_with_caller_info('info', msg)
+            def ok(self, msg):
+                self.log_with_caller_info('info', msg)
 
-        def teeprint(self, msg):
-            if not self.suppressPrintout:
-                printWithColor(msg, 'info')
-            self.log_with_caller_info('info', msg)
+            def teeprint(self, msg):
+                if not self.suppressPrintout:
+                    printWithColor(msg, 'info')
+                self.log_with_caller_info('info', msg)
 
-        def info(self, msg):
-            self.log_with_caller_info('info', msg)
+            def info(self, msg):
+                self.log_with_caller_info('info', msg)
 
-        def teeerror(self, msg):
-            if not self.suppressPrintout:
-                printWithColor(msg, 'error')
-            self.log_with_caller_info('error', msg)
+            def teeerror(self, msg):
+                if not self.suppressPrintout:
+                    printWithColor(msg, 'error')
+                self.log_with_caller_info('error', msg)
 
-        def error(self, msg):
-            self.log_with_caller_info('error', msg)
+            def error(self, msg):
+                self.log_with_caller_info('error', msg)
 
-        def teelog(self, msg, level):
-            if not self.suppressPrintout:
-                printWithColor(msg, level)
-            self.log_with_caller_info(level, msg)
+            def teelog(self, msg, level):
+                if not self.suppressPrintout:
+                    printWithColor(msg, level)
+                self.log_with_caller_info(level, msg)
 
-
-        def log(self, msg, level):
-            self.log_with_caller_info(level, msg)
+            def log(self, msg, level):
+                self.log_with_caller_info(level, msg)
 
 # ------------------------------
-def format_bytes(size, use_1024_bytes=True):
-    # size is in bytes
-    if use_1024_bytes:
-        power = 2**10
-        n = 0
-        power_labels = {0 : '', 1: 'Ki', 2: 'Mi', 3: 'Gi', 4: 'Ti', 5: 'Pi'}
-        while size > power:
-            size /= power
-            n += 1
-        return f"{size:.2f} {power_labels[n]}"
+def format_bytes(size, use_1024_bytes=None, to_int=False, to_str=False,str_format='.2f'):
+    """
+    Format the size in bytes to a human-readable format or vice versa.
+    From hpcp: https://github.com/yufei-pan/hpcp
+
+    Args:
+        size (int or str): The size in bytes or a string representation of the size.
+        use_1024_bytes (bool, optional): Whether to use 1024 bytes as the base for conversion. If None, it will be determined automatically. Default is None.
+        to_int (bool, optional): Whether to convert the size to an integer. Default is False.
+        to_str (bool, optional): Whether to convert the size to a string representation. Default is False.
+        str_format (str, optional): The format string to use when converting the size to a string. Default is '.2f'.
+
+    Returns:
+        int or str: The formatted size based on the provided arguments.
+
+    Examples:
+        >>> format_bytes(1500, use_1024_bytes=False)
+        '1.50 K'
+        >>> format_bytes('1.5 GiB', to_int=True)
+        1610612736
+        >>> format_bytes('1.5 GiB', to_str=True)
+        '1.50 Gi'
+        >>> format_bytes(1610612736, use_1024_bytes=True, to_str=True)
+        '1.50 Gi'
+        >>> format_bytes(1610612736, use_1024_bytes=False, to_str=True)
+        '1.61 G'
+    """
+    if to_int or isinstance(size, str):
+        if isinstance(size, int):
+            return size
+        elif isinstance(size, str):
+            # Use regular expression to split the numeric part from the unit, handling optional whitespace
+            match = re.match(r"(\d+(\.\d+)?)\s*([a-zA-Z]*)", size)
+            if not match:
+                if to_str:
+                    return size
+                print("Invalid size format. Expected format: 'number [unit]', e.g., '1.5 GiB' or '1.5GiB'")
+                print(f"Got: {size}")
+                return 0
+            number, _, unit = match.groups()
+            number = float(number)
+            unit  = unit.strip().lower().rstrip('b')
+            # Define the unit conversion dictionary
+            if unit.endswith('i'):
+                # this means we treat the unit as 1024 bytes if it ends with 'i'
+                use_1024_bytes = True
+            elif use_1024_bytes is None:
+                use_1024_bytes = False
+            unit  = unit.rstrip('i')
+            if use_1024_bytes:
+                power = 2**10
+            else:
+                power = 10**3
+            unit_labels = {'': 0, 'k': 1, 'm': 2, 'g': 3, 't': 4, 'p': 5}
+            if unit not in unit_labels:
+                if to_str:
+                    return size
+                print(f"Invalid unit '{unit}'. Expected one of {list(unit_labels.keys())}")
+                return 0
+            if to_str:
+                return format_bytes(size=int(number * (power ** unit_labels[unit])), use_1024_bytes=use_1024_bytes, to_str=True, str_format=str_format)
+            # Calculate the bytes
+            return int(number * (power ** unit_labels[unit]))
+        else:
+            try:
+                return int(size)
+            except Exception as e:
+                return 0
+    elif to_str or isinstance(size, int) or isinstance(size, float):
+        if isinstance(size, str):
+            try:
+                size = size.rstrip('B').rstrip('b')
+                size = float(size.lower().strip())
+            except Exception as e:
+                return size
+        # size is in bytes
+        if use_1024_bytes or use_1024_bytes is None:
+            power = 2**10
+            n = 0
+            power_labels = {0 : '', 1: 'Ki', 2: 'Mi', 3: 'Gi', 4: 'Ti', 5: 'Pi'}
+            while size > power:
+                size /= power
+                n += 1
+            return f"{size:{str_format}}{' '}{power_labels[n]}"
+        else:
+            power = 10**3
+            n = 0
+            power_labels = {0 : '', 1: 'K', 2: 'M', 3: 'G', 4: 'T', 5: 'P'}
+            while size > power:
+                size /= power
+                n += 1
+            return f"{size:{str_format}}{' '}{power_labels[n]}"
     else:
-        power = 10**3
-        n = 0
-        power_labels = {0 : '', 1: 'K', 2: 'M', 3: 'G', 4: 'T', 5: 'P'}
-        while size > power:
-            size /= power
-            n += 1
-        return f"{size:.2f} {power_labels[n]}"
+        try:
+            return format_bytes(float(size), use_1024_bytes)
+        except Exception as e:
+            import traceback
+            print(f"Error: {e}")
+            print(traceback.format_exc())
+            print(f"Invalid size: {size}")
+        return 0
 
 def almost_urandom(n):
     try:
-        return random.getrandbits(8 * n).to_bytes(n, 'big')
+        if numpy_available:
+            rng_sf = Generator(SFC64())
+            n_u64 = (n + 7) // 8
+            u64 = rng_sf.bit_generator.random_raw(n_u64)
+            return u64.view('uint8')[:n].tobytes()
+        else:
+            return random.getrandbits(8 * n).to_bytes(n, 'big')
     except OverflowError:
         return almost_urandom(n // 2) + almost_urandom(n - n // 2)
     
 def create_file(file_name, file_content,file_size,quiet=False,tl=None):
     if not tl:
-        tl = teeLogger(suppressPrintout=quiet)
+        tl = Tee_Logger.teeLogger(suppressPrintout=quiet)
     try:
         with open(file_name, "wb", buffering=0) as f:
             try:
@@ -138,7 +243,6 @@ def create_file(file_name, file_content,file_size,quiet=False,tl=None):
                     os.posix_fadvise(f.fileno(), 0, file_size, os.POSIX_FADV_DONTNEED)
             except:
                 tl.teelog(f'Failed to posix_fadvise, trying fallocate',level='warning')
-                pass
             start_write_time = time.perf_counter()
             try:
                 if os.name == 'posix':
@@ -159,7 +263,7 @@ def create_file(file_name, file_content,file_size,quiet=False,tl=None):
 
 def move_file(src, dst,tl=None):
     if not tl:
-        tl = teeLogger()
+        tl = Tee_Logger.teeLogger()
     start_move_time = time.perf_counter()
     try:
         os.rename(src, dst)
@@ -171,9 +275,9 @@ def move_file(src, dst,tl=None):
         tl.teeerror(traceback.format_exc())
         return 0,time.perf_counter()
 
-def read_file(file_name,file_content, file_size,zeros,quiet=False,tl=None):
+def read_file(file_name,file_content, file_size,quiet=False,tl=None):
     if not tl:
-        tl = teeLogger(suppressPrintout=quiet)
+        tl = Tee_Logger.teeLogger(suppressPrintout=quiet)
     b=bytearray(file_size)
     # check if file exists and size is correct
     try:
@@ -189,7 +293,6 @@ def read_file(file_name,file_content, file_size,zeros,quiet=False,tl=None):
                     os.posix_fadvise(f.fileno(), 0, file_size, os.POSIX_FADV_DONTNEED)
             except:
                 tl.teelog(f'Failed to posix_fadvise, trying fallocate',level='warning')
-                pass
             start_read_time = time.perf_counter()
             try:
                 if os.name == 'posix':
@@ -208,9 +311,9 @@ def read_file(file_name,file_content, file_size,zeros,quiet=False,tl=None):
         return 0,time.perf_counter()
     
 
-def index_file(file_name,zeros,quiet=False,tl=None):
+def index_file(file_name,quiet=False,tl=None):
     if not tl:
-        tl = teeLogger(suppressPrintout=quiet)
+        tl = Tee_Logger.teeLogger(suppressPrintout=quiet)
     try:
         # index creates file_name.index folder, stat it, then delete it
         # create the index folder
@@ -232,7 +335,7 @@ def index_file(file_name,zeros,quiet=False,tl=None):
     
 def stat_file(file_name,tl=None):
     if not tl:
-        tl = teeLogger()
+        tl = Tee_Logger.teeLogger()
     start_stat_time = time.perf_counter()
     try:
         size = os.stat(file_name).st_size
@@ -255,7 +358,7 @@ def int_to_color(n, brightness_threshold=500):
 
 def worker(file_count, file_size, directory, results, mode, counter,quiet,zeros,thread_start_time,tl=None):
     if not tl:
-        tl = teeLogger(suppressPrintout=quiet)
+        tl = Tee_Logger.teeLogger(suppressPrintout=quiet)
     local_results = []
     r, g, b = int_to_color(os.getpid())
     if not quiet:
@@ -276,8 +379,7 @@ def worker(file_count, file_size, directory, results, mode, counter,quiet,zeros,
     reportInterval = max(file_count // 10,1)
     for i in range(file_count):
         # print a digit every 10% of the files
-        if i % reportInterval == 0:
-            if quiet:
+        if i % reportInterval == 0 and quiet:
                 print(f'\r{i//reportInterval}',end='',flush=True)
         i = str(i+1).zfill(len(str(file_count)))
         file_name = os.path.join(directory,str(counter), f"temp_{i}.bin")
@@ -292,7 +394,7 @@ def worker(file_count, file_size, directory, results, mode, counter,quiet,zeros,
 
         elif mode == 'read':
             
-            start_read_time,end_read_time = read_file(file_name, file_content, file_size,zeros,quiet,tl=tl)
+            start_read_time,end_read_time = read_file(file_name, file_content, file_size,quiet,tl=tl)
 
             local_results.append(end_read_time- start_read_time)
             if not quiet:
@@ -342,7 +444,7 @@ def worker(file_count, file_size, directory, results, mode, counter,quiet,zeros,
 
             # then we read the file
 
-            start_read_time,end_read_time = read_file(file_name + ".moved",file_content, file_size,zeros,quiet,tl=tl)
+            start_read_time,end_read_time = read_file(file_name + ".moved",file_content, file_size,quiet,tl=tl)
 
             resultList.append(end_read_time- start_read_time)
 
@@ -352,7 +454,7 @@ def worker(file_count, file_size, directory, results, mode, counter,quiet,zeros,
             local_results.append(resultList)
         elif mode == 'index':
             # This will test the indexing performance of the filesystem
-            start_index_time,end_index_time = index_file(file_name, zeros,quiet,tl=tl)
+            start_index_time,end_index_time = index_file(file_name, quiet,tl=tl)
             local_results.append(end_index_time- start_index_time)
             if not quiet:
                 tl.teeprint(f'\033[38;2;{r};{g};{b}m' +f"[Process {os.getpid()}]\tFile {i}/{file_count}:\t{file_name}\tIndexed\tin {end_index_time-start_index_time} s"+ '\033[0m')
@@ -380,39 +482,79 @@ def addToDicWithoutOverwrite(dic,key,value):
     else:
         dic[key] = value
 
-
-def benchmarkGenSpeed(file_size, file_count,zeros,results):
+def benchmarkGenSpeed(file_size, file_count,zeros,results,timeout=5):
     start_time = time.perf_counter()
-    for i in range(file_count):
+    c = 0
+    while True:
         if zeros:
-            file_content = b'\x00' * file_size
+            _ = b'\x00' * file_size
         else:
-            file_content = almost_urandom(file_size) 
-        if time.perf_counter() - start_time > 5:
+            _ = almost_urandom(file_size) 
+        c += 1
+        if (file_count > 0 and c >= file_count) or time.perf_counter() - start_time > timeout:
             break
     genTime = time.perf_counter() - start_time
-    genSize = file_size * (i+1)
+    genSize = file_size * c
     genSpeed = genSize / genTime 
     results.append(genSpeed)
 
-def main(file_size, file_count, process_count, directory,modes,quiet,zeros,tl=None,stealth=False,message_end_point_address=None,no_report=False,threshold_to_report_anomaly = 0):
+def main(file_size, file_count, process_count, directory,modes,quiet,zeros,tl=None,stealth=False,message_end_point_address=None,no_report=False,threshold_to_report_anomaly = 0,benchmarkTime = 5):
     if stealth:
         quiet = True
         no_report = True
     if not tl:
-        tl = teeLogger(suppressPrintout=quiet)
+        tl = Tee_Logger.teeLogger(suppressPrintout=quiet)
     os.makedirs(directory, exist_ok=True)
     processes = []
     file_size = int(file_size)
 
     outResults = dict()
     totalTime = {}
+    estimatedTotalMemory = file_size * process_count * 1.2
+    phyFreeMemory = -1
+    swapMemory = -1
+    if modes == ['benchmark']:
+        file_count = 0
+    if os.path.exists('/proc/meminfo'):
+        try:
+            with open('/proc/meminfo') as f:
+                for line in f:
+                    if 'MemAvailable' in line:
+                        phyFreeMemory = int(line.split()[1]) * 1024
+                    if 'SwapFree' in line:
+                        swapMemory = int(line.split()[1]) * 1024
+        except:
+            tl.teeerror(f"Failed to read /proc/meminfo")
+    # warn if the estimated total memory is more than 90% of the total free memory
+    if phyFreeMemory + swapMemory > 0 and estimatedTotalMemory > phyFreeMemory + swapMemory:
+        tl.teeerror(f"Estimated total memory usage is more than the all available memory to use.")
+        tl.teeerror(f"Physical available memory: {format_bytes(phyFreeMemory)}B")
+        tl.teeerror(f"Swap available memory: {format_bytes(swapMemory)}B")
+        tl.teeerror(f"Estimated total memory usage: {format_bytes(estimatedTotalMemory)}B")
+        process_count = int((phyFreeMemory + swapMemory ) // file_size // 1.2)
+        tl.teeerror(f"Reducing the number of processes to {process_count}")
+    estimatedTotalMemory = file_size * process_count * 1.2
+    if phyFreeMemory > 0 and estimatedTotalMemory > phyFreeMemory * 0.9:
+        tl.teeerror(f"Estimated total memory usage is more than 90% of the total non swap available memory.")
+        tl.teeerror(f"You may want to reduce the file size or the number of processes.")
+        tl.teeerror(f"Available memory: {format_bytes(phyFreeMemory)}B")
+        tl.teeerror(f"Estimated total memory usage: {format_bytes(estimatedTotalMemory)}B")
+        tl.teelog(f"If continuing, iotest will likely put very heavy pressure on swap memory and may lead to system crash.",level='critical')
+        tl.teelog(f"Exit now (press Ctrl+C) or iotest will continue anyway in 15 seconds...",level='critical')
+        time.sleep(16)
+        tl.teelog(f"Warning! Continuing anyway... ",level='warning')
     with Manager() as manager:
         # bench mark file generation performance first
         results = manager.list()
-        tl.teeprint(f"Benchmarking file generation performance... with {format_bytes(file_size)}B {'zero' if zeros else 'random'} files")
+        if zeros:
+            generator = 'zeros'
+        elif numpy_available:
+            generator = 'numpy SFC64'
+        else:
+            generator = 'random.getrandbits'
+        tl.teeprint(f"Benchmarking file generation performance... using {process_count}x {generator} with {format_bytes(file_size)}B files * {file_count if file_count else '∞'} for {benchmarkTime}s")
         for counter in range(process_count):
-            p = Process(target=benchmarkGenSpeed, args=(file_size, file_count,zeros,results))
+            p = Process(target=benchmarkGenSpeed, args=(file_size, file_count,zeros,results,benchmarkTime))
             processes.append(p)
         genStartTime = time.perf_counter()
         for p in processes:
@@ -429,6 +571,8 @@ def main(file_size, file_count, process_count, directory,modes,quiet,zeros,tl=No
         tl.teeprint(f"Gen time calculated:   \t{genTimeCalc:.4f} s")
         tl.teeprint(f"Process start delay:   \t{processStartDelay:.4f} s")
         for mode in modes:
+            if mode == 'benchmark':
+                continue
             processes = []
             results = manager.list()
             thread_start_time = time.perf_counter() + processStartDelay
@@ -473,7 +617,7 @@ def main(file_size, file_count, process_count, directory,modes,quiet,zeros,tl=No
 
 
     # write the outResults to a csv file
-    if not no_report:
+    if not no_report and outResults:
         dir_str_repr = directory.replace('/','-').replace('\\','-').replace(':','-').replace('--','-').replace(' ','_')
         csv_file_name = os.path.join(directory, f"iotest_{'-'.join(modes)}_{dir_str_repr}_fs={file_size}_fc={file_count}_pc={process_count}_{tl.currentDateTime}.csv")
         with open(csv_file_name, "w") as f:
@@ -517,18 +661,18 @@ def main(file_size, file_count, process_count, directory,modes,quiet,zeros,tl=No
             total_size = file_size * file_count * process_count
             total_p_time = sum(outResults[mode]) / process_count
             total_bandwidth = total_size / total_p_time
-            report.append(f"Total {mode} size:     \t{format_bytes(total_size)}B")
-            report.append(f"Total {mode} speed:    \t{format_bytes(total_bandwidth)}B/s")
+            report.append(f"Total {mode} size:      \t{format_bytes(total_size)}B")
+            report.append(f"{mode} bandwidth (call):\t{format_bytes(total_bandwidth)}B/s")
             report.append(f"                       \t{format_bytes(total_bandwidth * 8,use_1024_bytes=False)}b/s")
-            report.append(f"With generation time:  \t{format_bytes(total_size / totalTime[mode])}B/s")
+            report.append(f"{mode} bandwidth (sync):\t{format_bytes(total_size / totalTime[mode])}B/s")
             report.append(f"                       \t{format_bytes(total_size / totalTime[mode] * 8,use_1024_bytes=False)}b/s")
-            report.append(f"IO time %:             \t{total_p_time / totalTime[mode] * 100:.2f}%")
-        report.append(f"Average {mode} time:   \t{avg_time:.4f} s")
-        report.append(f"Median {mode} time:    \t{median_time:.4f} s")
-        report.append(f"1 % low {mode} time:   \t{one_percent_low_time:.4f} s")
-        report.append(f"1 % high {mode} time:  \t{one_percent_high_time:.4f} s")
-        report.append(f"0.1 % high {mode} time:\t{zero_point_one_percent_high_time:.4f} s")
-        report.append(f"Highest {mode} time:   \t{highest_time:.4f} s")
+            report.append(f"IO time % (IO wait):   \t{total_p_time / totalTime[mode] * 100:.2f}%")
+        report.append(f"Average {mode} time:    \t{avg_time:.4f} s")
+        report.append(f"Median {mode} time:     \t{median_time:.4f} s")
+        report.append(f"1 % low {mode} time:    \t{one_percent_low_time:.4f} s")
+        report.append(f"1 % high {mode} time:   \t{one_percent_high_time:.4f} s")
+        report.append(f"0.1 % high {mode} time: \t{zero_point_one_percent_high_time:.4f} s")
+        report.append(f"Highest {mode} time:    \t{highest_time:.4f} s")
         report.append(f"-"*80)
         if threshold_to_report_anomaly and threshold_to_report_anomaly > 0 and one_percent_high_time > (one_percent_low_time * threshold_to_report_anomaly):
             report.append(f"Warning | iotest: 1% high is too high compared to 1% low!")
@@ -544,7 +688,7 @@ def main(file_size, file_count, process_count, directory,modes,quiet,zeros,tl=No
     tl.teeprint('\n'.join(report))
 
     # write the report to a file
-    if not no_report:
+    if not no_report and outResults:
         report_file_name = os.path.join(directory, f"iotest_{'-'.join(modes)}_{dir_str_repr}_fs={file_size}_fc={file_count}_pc={process_count}_{tl.currentDateTime}_report.txt")
         with open(report_file_name, "w") as f:
             f.write('\n'.join(report))
@@ -561,11 +705,11 @@ def climain():
     parser.add_argument('-d',"--directory", type=str, help="Directory to put the files in (default:<pwd>)", default=os.getcwd())
     parser.add_argument('-ld',"--log_directory", type=str, help="Directory to put the log files in (default:/var/log/)", default='/var/log/')
     parser.add_argument("mode", nargs='*', type=str, help="""The mode the script will operate in (default:comprehensive).
- COMPREHENSIVE: async fully cached per thread write - index - read operation.
+ COMPREHENSIVE: async fully cached per thread write - index - read operation (what your code see).
  WRITE: batched all thread write.
  READ: batched all thread read.
  INDEX: creates --file_count amount of index folders, stat it, then delete it.
- RWI: Execute Write - Index - Read mode sequentially in batch mode.""",choices=['comprehensive','read', 'write','random','index','r','w','rw','wr','i','rwi','wri','c'], default="c")
+ RWI: Execute Write - Index - Read mode sequentially in batch mode.""",choices=['read', 'write','random','index','benchmark','comprehensive','r','w','rw','wr','i','rwi','wri','c','b'], default="w")
     parser.add_argument("-q","--quiet", action="store_true", help="Suppress output, default True in new version",default=True)
     parser.add_argument("-v","--verbose", action="store_true", help="Verbose output",default=False)
     parser.add_argument("-S",'--stealth', action="store_true", help="Suppress verbose output and verbose log file",default=False)
@@ -574,7 +718,8 @@ def climain():
     parser.add_argument("-z","--zeros", action="store_true", help="Use zeros instead of random numbers. Use this if you are sure no write compression is available. Potentially higher write accuracy.",default=False)
     parser.add_argument('-addr',"--message_end_point_address", type=str, help="The end point address of the message")
     parser.add_argument('--threshold_to_report_anomaly', type=int, help="The threshold to report if 1 percent high is higher then 1 percent low * <threshold_to_report_anomaly>",default=0)
-    parser.add_argument("-V","--version", action="version", version=f"%(prog)s {version} with teeLogger {teeLogger().version} by pan@zopyr.us")
+    parser.add_argument('-bt',"--benchmark_time", type=int, help="The time in seconds to run the benchmark for file generation speed",default=5)
+    parser.add_argument("-V","--version", action="version", version=f"%(prog)s {version} with teeLogger {Tee_Logger.version} {' and numpy ' if numpy_available else ''} by pan@zopyr.us")
     args = parser.parse_args()
 
     # if we are on windows, set the log directory to the current directory
@@ -588,8 +733,8 @@ def climain():
         args.no_report = True
     if args.no_log:
         args.log_directory = '/dev/null'
-    #tl = teeLogger(args.log_directory,'iotest',2,10)
-    tl = teeLogger(systemLogFileDir=args.log_directory,programName='iotest',compressLogAfterMonths=1,deleteLogAfterYears=3,suppressPrintout=args.stealth,noLog=args.no_log)
+    #tl = Tee_Logger.teeLogger(args.log_directory,'iotest',2,10)
+    tl = Tee_Logger.teeLogger(systemLogFileDir=args.log_directory,programName='iotest',compressLogAfterMonths=1,deleteLogAfterYears=3,suppressPrintout=args.stealth,noLog=args.no_log,in_place_compression=True)
     
     tl.info(f'Arguments: {vars(args)}')
 
@@ -610,6 +755,8 @@ def climain():
             modes.append('index')
         elif mode == 'c':
             modes.append('comprehensive')
+        elif mode == 'b':
+            modes.append('benchmark')
     
         if 'write' in mode:
             modes.append('write')
@@ -621,6 +768,8 @@ def climain():
             modes.append('index')
         if 'comprehensive' in mode:
             modes.append('comprehensive')
+        if 'benchmark' in mode:
+            modes.append('benchmark')
 
     args.file_size = args.file_size.lower()
     if 'm' in args.file_size.lower():
@@ -646,14 +795,16 @@ def climain():
         tl.info(f'File size: {format_bytes(args.file_size)}B')
         tl.info(f'Number of files: {args.file_count}')
         tl.info(f'Number of processes: {args.process_count}')
-        tl.info(f'Writing to {args.directory}')
+        if 'benchmark' not in modes and len(modes) == 1:
+            tl.info(f'Writing to {args.directory}')
     else:
         tl.teeprint(f'Running in {modes} modes...')
         tl.teeprint(f'File size: {format_bytes(args.file_size)}B')
         tl.teeprint(f'Number of files: {args.file_count}')
         tl.teeprint(f'Number of processes: {args.process_count}')
-        tl.teeprint(f'Writing to {args.directory}')
-    main(args.file_size, args.file_count, args.process_count, args.directory,modes,args.quiet,args.zeros,tl=tl,stealth=args.stealth,message_end_point_address=args.message_end_point_address,no_report=args.no_report,threshold_to_report_anomaly=args.threshold_to_report_anomaly)
+        if 'benchmark' not in modes and len(modes) == 1:
+            tl.teeprint(f'Writing to {args.directory}')
+    main(args.file_size, args.file_count, args.process_count, args.directory,modes,args.quiet,args.zeros,tl=tl,stealth=args.stealth,message_end_point_address=args.message_end_point_address,no_report=args.no_report,threshold_to_report_anomaly=args.threshold_to_report_anomaly,benchmarkTime=args.benchmark_time)
                 
 if __name__ == "__main__":
     climain()
\ No newline at end of file
